%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generate_sparsecov:
%
% Generates a sparse covariance matrix using the bi-directed structure
% encoded by symmetric graph G.
%
% This returns three items:
%
% - Sigma: the (sparse) covariance matrix
% - B, E: the covariance is generated by a sparse factor analysis model.
%         Let X be a set of independent standard Gaussians (one for each 
%         bi-directed edge in the lower triangular part of G). Sigma is
%         a function of loadings B and diagonal covariance E.
%
% Initial version: Ricardo Silva, UCL - London, April 2011
% Current version: 1.0 (04/04/2011)

function [Sigma B E] = generate_sparsecov(G)

num_y = size(G, 1);
num_x = sum(sum(G)) / 2;
B = zeros(num_y, num_x);
x_count = 1;
for y1 = 1:num_y
  for y2 = (y1 + 1):num_y
    if G(y1, y2) == 1
      B(y1, x_count) = simulate_coeff();
      B(y2, x_count) = simulate_coeff();
      x_count = x_count + 1;
     end
  end
end

E = diag(diag(B * eye(num_x) * B' / 10));
for y = 1:num_y
    if E(y, y) == 0
       E(y, y) = 1;
    end
end
Sigma = B * eye(num_x) * B' + E;
