% get_districts:
%
% Returns a cell structure listing the districts of the graph.
%
% Moreover, vector 'dis_x' specifies, for each vertex 'x', the index of the
% corresponding district in the cell structure 'districts'. Vector entry
% 'dis_pos(x)' indicates the position of 'x' within its district vector
% 'districts{dis_x{x}}'.
%
% 'code' indicates which values represent bi-directed edges.


function [districts dis_x dis_pos] = get_districts(graph, code)

num_x = size(graph, 1);
districts = cell(0);
dis_x = zeros(num_x, 1);
dis_pos = zeros(num_x, 1);

remaining_x = 1:num_x;
num_districts = 0;

while ~isempty(remaining_x)
    queue = remaining_x(1);
    num_districts = num_districts + 1;
    districts{num_districts} = [];    
    while ~isempty(queue)
        x_seed = queue(1);
        remaining_x = setdiff(remaining_x, x_seed);
        districts{num_districts} = [districts{num_districts} x_seed];
        dis_x(x_seed) = num_districts;
        dis_pos(x_seed) = length(districts{num_districts});
        other_sp = setdiff(intersect(find(graph(x_seed, :) == code), remaining_x), queue);
        queue = [queue(2:end) other_sp];
    end
end
